﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.ComponentModel.Composition;
using PluginContracts;
using DataConnection;
using static DAMReader;
using System.Reflection;

namespace TunerNerd
{
    [Export(typeof(IDataIOProvider))]

    public partial class TunerNerd : IDataIOProvider
    {

        DAMReader damReader = new DAMReader();

        private OnePlugInDataConnection AFRv = new OnePlugInDataConnection();
        private OnePlugInDataConnection RPM = new OnePlugInDataConnection();
        private OnePlugInDataConnection MAP = new OnePlugInDataConnection();
        private OnePlugInDataConnection rawLevel = new OnePlugInDataConnection();
        private OnePlugInDataConnection avgLevel = new OnePlugInDataConnection();
        private OnePlugInDataConnection rawLevelLeft = new OnePlugInDataConnection();
        private OnePlugInDataConnection rawLevelRight = new OnePlugInDataConnection();
        private OnePlugInDataConnection DutyC = new OnePlugInDataConnection();
        private OnePlugInDataConnection threshold = new OnePlugInDataConnection();
        private OnePlugInDataConnection outlev = new OnePlugInDataConnection();
        private OnePlugInDataConnection distance = new OnePlugInDataConnection();
        private OnePlugInDataConnection knockCountL = new OnePlugInDataConnection();
        private OnePlugInDataConnection knockCountR = new OnePlugInDataConnection();

        public event ConfigChangeEventHandler OnConfigurationChange; // not needed for this module
        private TunerNerdGUI tunerNerdGUI;

        public ToolStripMenuItem pluginMenuEntry { get; } = new ToolStripMenuItem();
        public List<OnePlugInDataConnection> pluginDataConnections { get; } = new List<OnePlugInDataConnection>();

        public List<Keys> hotkeys
        {
            get
            {
                return null;
            }
        }

        public void hotkeyPressed(Keys hotkey)
        {
        }
        public string name
        {
            get
            {
                return "TunerNerd";
            }
        }

        public string pluginDescription
        {
            get
            {
                return @"Provides an interface to the TunerNerd Knock monitor. It uses the serial connection";
            }
        }

        public string version
        {
            get
            {
                return "0.1";
            }
        }

        private void menuItem_Click(object sender, EventArgs e)
        {
            tunerNerdGUI.StartPosition = FormStartPosition.CenterParent;
            tunerNerdGUI.Show();
        }

        public void initDynoDataConnection(DynoDataConnection d, Dictionary<string, OnePlugInDataConnection> p)
        {
            // not used
        }

        public void TunerNerdDataReceived(object sender, EventArgs e)
        {
            MAP.y = (float)damReader.GetMAP();
            RPM.y = (float)damReader.GetRPM();
            rawLevel.y = (float)damReader.GetRaw();
            avgLevel.y = (float)damReader.GetSND();
            rawLevelLeft.y = (float)damReader.GetSNDL();
            rawLevelRight.y = (float)damReader.GetSNDR();
            DutyC.y = (float)damReader.GetDuty();
            threshold.y = (float)damReader.GetThresh();
            outlev.y = (float)damReader.GetOutLevel();
            distance.y = (float)damReader.GetDistance();
            knockCountL.y = (float)damReader.GetKnockCountL();
            knockCountR.y = (float)damReader.GetKnockCountR();
        }
    
        public TunerNerd()
        {
            damReader.dataReceivedEvent += TunerNerdDataReceived;
            tunerNerdGUI = new TunerNerdGUI(damReader);

            pluginMenuEntry.Name = "menuEntry";
            pluginMenuEntry.Text = "TunerNerd";
            pluginMenuEntry.Click += new EventHandler(menuItem_Click);

            MAP.name = "MAP[TunerNerd]";
            MAP.unit = "";
            MAP.graphPane = 1;
            MAP.isY2Axis = true;
            MAP.showGaugeInRunWindow = true;
            MAP.applyNoiseFiltering = true;
            pluginDataConnections.Add(MAP);

            RPM.name = "RPM[TunerNerd]";
            RPM.unit = "";
            RPM.graphPane = 1;
            RPM.isY2Axis = true;
            RPM.showGaugeInRunWindow = true;
            RPM.applyNoiseFiltering = true;
            pluginDataConnections.Add(RPM);

            rawLevel.name = "RawLevel[TunerNerd]";
            rawLevel.unit = "";
            rawLevel.graphPane = 1;
            rawLevel.isY2Axis = true;
            rawLevel.showGaugeInRunWindow = true;
            rawLevel.applyNoiseFiltering = true;
            pluginDataConnections.Add(rawLevel);

            avgLevel.name = "AvgLevel[TunerNerd]";
            avgLevel.unit = "";
            avgLevel.graphPane = 1;
            avgLevel.isY2Axis = true;
            avgLevel.showGaugeInRunWindow = true;
            avgLevel.applyNoiseFiltering = true;
            pluginDataConnections.Add(avgLevel);

            rawLevelLeft.name = "RawLevelLeft[TunerNerd]";
            rawLevelLeft.unit = "";
            rawLevelLeft.graphPane = 1;
            rawLevelLeft.isY2Axis = true;
            rawLevelLeft.showGaugeInRunWindow = true;
            rawLevelLeft.applyNoiseFiltering = true;
            pluginDataConnections.Add(rawLevelLeft);

            rawLevelRight.name = "rawLevelRight[TunerNerd]";
            rawLevelRight.unit = "";
            rawLevelRight.graphPane = 1;
            rawLevelRight.isY2Axis = true;
            rawLevelRight.showGaugeInRunWindow = true;
            rawLevelRight.applyNoiseFiltering = true;
            pluginDataConnections.Add(rawLevelRight);

            DutyC.name = "DutyC[TunerNerd]";
            DutyC.unit = "";
            DutyC.graphPane = 1;
            DutyC.isY2Axis = true;
            DutyC.showGaugeInRunWindow = true;
            DutyC.applyNoiseFiltering = true;
            pluginDataConnections.Add(DutyC);

            threshold.name = "Threshold[TunerNerd]";
            threshold.unit = "";
            threshold.graphPane = 1;
            threshold.isY2Axis = true;
            threshold.showGaugeInRunWindow = true;
            threshold.applyNoiseFiltering = true;
            pluginDataConnections.Add(threshold);

            outlev.name = "Out level[TunerNerd]";
            outlev.unit = "";
            outlev.graphPane = 1;
            outlev.isY2Axis = true;
            outlev.showGaugeInRunWindow = true;
            outlev.applyNoiseFiltering = true;
            pluginDataConnections.Add(outlev);

            distance.name = "Distance[TunerNerd]";
            distance.unit = "";
            distance.graphPane = 1;
            distance.isY2Axis = true;
            distance.showGaugeInRunWindow = true;
            distance.applyNoiseFiltering = true;
            pluginDataConnections.Add(distance);

            knockCountL.name = "KnockCountL[TunerNerd]";
            knockCountL.unit = "";
            knockCountL.graphPane = 1;
            knockCountL.isY2Axis = true;
            knockCountL.showGaugeInRunWindow = true;
            knockCountL.applyNoiseFiltering = true;
            pluginDataConnections.Add(knockCountL);

            knockCountR.name = "KnockCountR[TunerNerd]";
            knockCountR.unit = "";
            knockCountR.graphPane = 1;
            knockCountR.isY2Axis = true;
            knockCountR.showGaugeInRunWindow = true;
            knockCountR.applyNoiseFiltering = true;
            pluginDataConnections.Add(knockCountR);
        }

    }
}
